**************************
****Chapter 13: Counts****
**************************


*** 13.4 Did the German health-care reform reduce the number of doctor visits?

use http://www.stata-press.com/data/mlmus3/drvisits, clear


*** 13.5 Longitudinal data structure

quietly xtset id reform
xtdescribe if numvisit<.


*** 13.6 Single-level Poisson regression
poisson numvisit reform age educ married badh loginc summer, irr vce(cluster id)
estimates store ordinary 

glm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) vce(cluster id) eform


*** 13.7 Random-intercept Poisson regression
** 13.7.3 Estimation using Stata 
* Using xtpoisson
quietly xtset id
xtpoisson numvisit reform age educ married badh loginc summer, ///
 normal irr 

display exp(sqrt(2)*.90407*invnormal(3/4))

* Using xtmepoisson
xtmepoisson numvisit reform age educ married badh loginc summer ///
  || id:, irr
estimates store xtri

* Using gllamm
generate cons = 1
eq ri: cons
gllamm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) i(id) eqs(ri) eform adapt 
estimates store glri

display sqrt(.81691979)

gllamm, robust eform

lrtest ordinary glri, force


*** 13.8 Random-coefficient Poisson regression 

** 13.8.2 Estimation using Stata
* Using xtmepoisson
xtmepoisson numvisit reform age educ married badh loginc summer ///
  || id: reform, covariance(unstructured) irr
estimates store xtrc

lrtest xtri xtrc

* Using gllamm
estimates restore glri
matrix a = e(b)
eq rc: reform
gllamm numvisit reform age educ married badh loginc summer, ///
  family(poisson) link(log) i(id) nrf(2) eqs(ri rc) from(a) eform adapt 
estimates store glrc
estimates save glrc, replace

gllamm, robust eform


*** 13.9 Overdispersion in single-level models

** 13.9.1 Normally distributed random intercept 
quietly xtset obs
generate obs = _n
xtpoisson numvisit reform age educ married badh loginc summer, ///
   normal irr 

*** 13.9.2 Negative binomial models
* Mean dispersion or NB2
nbreg numvisit reform age educ married badh loginc summer, irr ///
   dispersion(mean)
* Constant dispersion or NB1
nbreg numvisit reform age educ married badh loginc summer, irr ///
   dispersion(constant)

*** 13.9.3 Quasilikelihood or robust standard errors 
glm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) eform scale(x2)


*** 13.10 Level-1 overdispersion in two-level models

*xtset id
*xtnbreg numvisit reform age educ married badh loginc summer, irr

estimates restore glri

gllamm, robust eform


*** 13.11 Other approaches to two-level count data

** 13.11.1 Conditional Poisson regression

quietly xtset id
xtpoisson numvisit reform educ married badh loginc summer, ///
  fe irr
  
*set matsize 2000

poisson numvisit reform educ married badh loginc summer i.id, irr

** 13.11.2 Conditional negative binomial regression

*xtnbreg numvisit reform age educ married badh loginc summer, fe irr

** 13.11.3 Generalized estimating equations (GEE)

quietly xtset id
xtgee numvisit reform age educ married badh loginc summer, ///
  family(poisson) link(log) vce(robust) eform
  
estat wcorrelation
 
** 13.12 Estimating marginal and conditional effects when responses are missing at random

egen num = count(numvisit), by(id)
generate mult = 3 - num
expand mult
by id (reform), sort: generate occ = _n - 1
generate missing = (occ!=reform & num==1)
replace numvisit = . if missing==1
replace reform = occ if missing==1

sort id reform
set seed 1211

estimates restore glri
gllasim y, fsample

by id (reform), sort: generate drop = (y[1]>2 & uniform()<.9)
replace y = . if drop==1 & reform==1

quietly xtset id
xtpoisson y reform age educ married badh loginc summer, ///
 normal irr 
 
poisson y reform age educ married badh loginc summer, irr

quietly xtset id
xtgee y reform age educ married badh loginc summer, ///
  family(poisson) link(log) vce(robust) eform



*** 13.13 Which Scottish counties have a high risk of lip cancer?

use http://www.stata-press.com/data/mlmus3/lips, clear


*** 13.14 Standardized mortality ratios

generate smr = 100*o/e


*** 13.15 Random-intercept Poisson regression

** 13.15.2 Estimation using gllamm

generate lne = ln(e)
gllamm o, i(county) offset(lne) family(poisson) link(log) adapt

** 13.15.3 Prediction of SMRs

gllapred mu, mu nooffset
generate thet = 100*mu
sort county
list county thet in 1/10, clean noobs

*copy http://www.stata-press.com/data/mlmus3/scotmaps.do scotmaps.do

twoway (scatter thet smr, msymbol(none) mlabpos(0) mlabel(county)) ///
 (function y=x, range(0 600)), xline(108) yline(108)               ///
 xtitle(Crude SMR) ytitle(Empirical Bayes SMR) legend(off)
 

*** 13.16 Nonparametric maximum likelihood estimation

** 13.16.2 Estimation using gllamm
gllamm o, i(county) offset(lne) family(poisson) link(log) ip(f) nip(2) 

matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) link(log) ip(f) nip(3) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')

matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) link(log) ip(f) nip(4) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')
  
matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) link(log) ip(f) nip(5) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')
  
matrix locs = e(zlc2)'
matrix lp = e(zps2)'
svmat locs
svmat lp

generate p = exp(lp1)
generate smrloc = 100*exp(_b[_cons] + locs1)
twoway (dropline p smrloc), xtitle(Location) ytitle(Probability)

** 9.16.3 Prediction
gllapred mu2, mu nooffset
generate thet2 = 100*mu2
sort county
list county thet thet2 in 1/10, clean noobs

