**********************************************
****Chapter 15: Continuous-time survival *****
**********************************************


*** 15.2 What makes marriages fail?

use http://www.stata-press.com/data/mlmus3/divorce, clear

generate mixrace = (heblack & !sheblack) | (sheblack & !heblack)
generate hedropout = hiseduc<12
generate hecollege = hiseduc>=16 if hiseduc<.
generate heolder = agediff > 10 if agediff<.
generate sheolder = agediff< -10


*** 15.3 Hazards and survival

list id dur divorce in 1/4, clean noobs

stset dur, failure(divorce) id(id)

sts graph

sts graph, hazard


*** 15.4 Proportional hazards models

** 15.4.1. Piecewise exponential model

stptime, at(0,1,5,9,15,25)

stsplit interval, at(1,5,9,15,25)
sort id _t
list id interval _t0 _t _d if id<34, sepby(id) noobs
 
generate pers_yr = _t-_t0
table interval, contents(freq sum pers_yr sum _d)

* piecewise exponential
streg ibn.interval, distribution(exponential) noconstant

streg ibn.interval heblack mixrace hedropout hecollege heolder sheolder, ///
   distribution(exponential) noconstant

generate h0 = exp(_b[0.interval]*(interval==0) + _b[1.interval]*(interval==1)  ///
   + _b[5.interval]*(interval==5) + _b[9.interval]*(interval==9) ///
   + _b[15.interval]*(interval==15) + _b[25.interval]*(interval==25))
 
twoway line h0 _t, connect(J) sort xtitle(Time since wedding in years) ///
   ytitle(Baseline hazard) 


* Poisson trick
generate lexposure = ln(_t - _t0)
poisson _d ibn.interval heblack mixrace hedropout hecollege ///
   heolder sheolder, offset(lexposure) noconstant irr
   
drop interval pers_yr lexposure h0
stjoin


** 15.4.2 Cox regression model

stcox heblack mixrace hedropout hecollege heolder sheolder

stcurve, hazard xtitle(Time since wedding in years) at1(sheolder=0) at2(sheolder=1)

stcurve, hazard xtitle(Time since wedding in years) ///
   at(heblack=0 mixrace=0 hedropout=0 hecollege=0 heolder=0 sheolder=0) ///
   outfile(temp, replace)

* split the data
stsplit, at(failures) riskset(click)
list id click _t0 _t _d _st if id==9 & (click<5 | click>624), separator(4) noobs

generate lexposure = ln(_t - _t0)
quietly xtset click
xtpoisson _d heblack mixrace hedropout hecollege heolder sheolder, ///
   fe offset(lexposure) irr

*clogit d heblack mixrace hedropout hecollege heolder sheolder, group (click)
   
mkspline sp = _t, cubic knots(1,5,7,9,12,15,20,25)
poisson _d sp* heblack mixrace hedropout hecollege heolder sheolder, ///
   offset(lexposure) irr

matrix a=e(b)
matrix coeff = a[1,1..7],a[1,14..14]
matrix score lhazard = coeff
generate h0=exp(lhazard)

append using temp
twoway (line h0 _t if divorce<., sort)  (line haz1 _t , sort), ///
   xtitle(Time since wedding in years) ///
   ytitle(Baseline hazard) legend(order(1 "Cubic spline" 2 "Cox"))

drop if _st >= .
drop sp* h0 haz1 lexposure lhazard click
stjoin


*** 15.5 Accelerated failure time models

** 15.5.1 Log-normal model

streg heblack mixrace hedropout hecollege heolder sheolder,  ///
    distribution(lognormal) time
	
streg, tr

stcurve, hazard xtitle(Time since wedding in years) at1(sheolder=0) at2(sheolder=1)

generate ldur = ln(dur)
generate ldur2 = ldur if divorce==1 

intreg ldur ldur2 heblack mixrace hedropout hecollege heolder sheolder

   

*** 15.6 Time-varying covariates

use http://www.stata-press.com/data/mlmus3/divorce2, clear

list id dur numkids divorce if id<34, sepby(id) noobs

stset dur, failure(divorce) id(id)

sort id dur
list id _t0 _t numkids divorce if id<34, sepby(id) noobs

generate mixrace = (heblack & !sheblack) | (sheblack & !heblack)
generate hedropout = hiseduc<12
generate hecollege = hiseduc>=16 if hiseduc<.
generate heolder = agediff > 10 if agediff<.
generate sheolder = agediff< -10

streg heblack mixrace hedropout hecollege heolder sheolder  ///
    numkids, distribution(lognormal) time tr


	
*** 15.7 Does nitrate reduce the hazard of angina pectoris?

use angina8, clear
list subj regime occ second uncen in 1/16, sepby(subj) noobs

tabulate occ, generate(occ)
generate treat = regime==1 & occ>1
generate id = _n
list id subj regime occ2 occ3 occ4 treat second uncen in 1/16, sepby(subj) noobs

stset second, failure(uncen) id(id)


*** 15.8 Marginal modeling

** 15.8.1 Cox regression

stcox occ2 occ3 occ4 treat, vce(cluster subj)

stcurve, hazard at1(occ2=1 occ3=0 occ4=0 treat=1) ///
  at1(occ2=1 occ3=0 occ4=0 treat=0) legend(order(1 "Treatment" 2 "Placebo")) ///
  xtitle(Time in seconds) ytitle(Hazard function) 

stcurve, hazard name(cox, replace) at(occ2=0 occ3=0 occ4=0 treat=0) ///
  outfile(temp, replace)

stcox treat, strata(occ) vce(cluster subj)


** 15.8.2 Poisson regression with smooth baseline hazard

stsplit, at(failures) riskset(click)

generate lny = ln(_t - _t0)

xtpoisson _d occ2 occ3 occ4 treat, i(click) fe offset(lny) irr

orthpoly _t, gen(t1-t4) degree(4)
poisson _d t1-t4 occ2 occ3 occ4 treat, offset(lny) irr vce(cluster subj)

matrix a = e(b)
matrix coeff = a[1,1..4], a[1,9..9]
matrix score lhazard = coeff
generate haz0 = exp(lhazard)

append using temp
twoway (line haz0 _t, sort) (line haz1 _t, sort), ///
   xtitle(Time in seconds) ytitle(Baseline hazard function) ///
   legend(order(1 "Polynomial" 2 "Cox"))



*** 15.9 Multilevel proportional hazards models

** 15.9.1 Cox regression with gamma shared frailty

drop click lny lhazard haz0 haz1 t1-t4
stjoin

stcox occ2 occ3 occ4 treat, frailty(gamma) shared(subj)

stcurve, at1(occ2=1 occ3=0 occ4=0 treat=1) at2(occ2=1 occ3=0 occ4=0 treat=0) ///
   hazard xtitle(Time in seconds) ytitle(Hazard function) ///
   legend(order(1 "Treatment" 2 "Placebo")) range(87 430)
   
*xtset subj
*xtpoisson _d i.click occ2 occ3 occ4 treat, re offset(lny) irr 


** 15.9.2 Poisson regression with normal random intercept

stsplit, at(failures) riskset(click)
generate lny = ln(_t - _t0)
orthpoly _t, gen(t1-t4) degree(4)

quietly xtset subj
xtpoisson _d t1-t4 occ2 occ3 occ4 treat, re normal offset(lny) irr

display exp(sqrt(2*2.078079^2)*invnormal(3/4))


** 15.9.3 Poisson regression with normal random intercept and random coefficient

xtmepoisson _d t1-t4 occ2 occ3 occ4 treat || subj: treat, ///
   covariance(unstructured) offset(lny) irr
estimates save xtmepoisson, replace
   
drop lny click t1-t4
stjoin


*** 15.10 Multilevel accelerated failure time models

** 15.10.1 Lognormal model with gamma shared frailty

streg occ2 occ3 occ4 treat, distribution(lognormal) shared(subj) frailty(gamma)

stcurve, at1(occ2=1 occ3=0 occ4=0 treat=1) ///
   at2(occ2=1 occ3=0 occ4=0 treat=0) hazard ///
   xtitle(Time in seconds) ytitle(Hazard function) ///
   legend(order(1 "Treatment" 2 "Placebo")) 
   
stcurve, at1(occ2=1 occ3=0 occ4=0 treat=1) ///
   at2(occ2=1 occ3=0 occ4=0 treat=0) hazard ///
   xtitle(Time in seconds) ytitle(Hazard function) ///
   legend(order(1 "Treatment" 2 "Placebo")) unconditional

   
** 15.10.2 Lognormal model with lognormal shared frailty

generate lnsecond = ln(second)
generate dep_lo = lnsecond
generate dep_hi = lnsecond if uncen == 1
replace dep_hi = . if uncen == 0

quietly xtset subj
xtintreg dep_hi dep_lo occ2 occ3 occ4 treat


*** 15.11 A fixed-effects approach

stcox occ2 occ3 occ4 treat, strata(subj)


*** 15.12 Different approaches to recurrent event data

use http://www.stata-press.com/data/mlmus3/recurrent, clear

list, sepby(id) noobs
egen idnum = group(id number)

** 15.12.1 Total time

* unrestricted risk sets
stset stop, failure(event) id(idnum)
list id number stop event _t0 _t _d if _st==1, sepby(id) noobs

* semi-restricted risk sets
fillin id number
by id (number), sort: replace stop = stop[_n-1] if _fillin==1
replace event = 0 if _fillin==1

drop idnum
egen idnum = group(id number)

stset stop, failure(event) id(idnum)
list id number stop event _t0 _t _d if _st==1, sepby(id) noobs

*** 15.12.2 Counting process

by id (number), sort: generate start = stop[_n-1] if _n>1
replace start = 0 if start==.

stset stop, failure(event) enter(start) id(idnum)

list id number stop event _t0 _t _d if _st==1, sepby(id) noobs

*** 15.12.3 Gap time

stset stop, failure(event) origin(start) id(idnum)
list id number stop event _t0 _t _d if _st==1, sepby(id) noobs






