************************************************************
****Chapter 3: Random-intercept models with covariates *****
************************************************************


*** 3.2 Does smoking during pregnancy affect birthweight?

** 3.2.1 Data structure and descriptive statistics

use http://www.stata-press.com/data/mlmus3/smoking, clear

quietly xtset momid 
xtsum birwt smoke black

egen pickone = tag(momid)
summ black if pickone==1

egen num = count(birwt), by(momid)
tabulate num if pickone==1

quietly xtset momid
xttab smoke


*** 3.4 Estimation using Stata

** 3.4.1 Using xtreg
quietly xtset momid
xtreg birwt smoke male mage hsgrad somecoll collgrad       ///
  married black kessner2 kessner3 novisit pretri2 pretri3, mle 

** 3.4.2 Using xtmixed

xtmixed birwt smoke male mage hsgrad somecoll collgrad     ///
  married black kessner2 kessner3 novisit pretri2 pretri3  ///
  || momid:, mle


*** 3.5 Coefficients of determination or variance explained

quietly xtset momid
xtreg birwt, mle

quietly xtset momid
xtreg birwt hsgrad somecoll collgrad married black, mle


*** 3.6 Hypothesis tests

** 3.6.1 Hypothesis tests for regression coefficients

* Wald test
quietly xtset momid
quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3, mle
testparm kessner2 kessner3

* Likelihood-ratio test
estimates store full
quietly xtset momid
quietly xtreg birwt smoke male mage hsgrad somecoll collgrad   ///
  married black novisit pretri2 pretri3, mle
lrtest full .

** 3.6.2 Predicted means and CIs
generate education = hsgrad*1 + somecoll*2 + collgrad*3

quietly xtset momid
quietly xtreg birwt i.smoke male mage i.education    ///
  married black kessner2 kessner3 novisit pretri2 pretri3, mle
  
margins i.smoke#i.education 

marginsplot, xdimension(education)


*** 3.7 Between and within effects

** 3.7.1 Between-mother effects

quietly xtset momid
xtreg birwt smoke male mage hsgrad somecoll collgrad       ///
  married black kessner2 kessner3 novisit pretri2 pretri3, be
  
** 3.7.2 Within-mother effects

quietly xtset momid
xtreg birwt smoke male mage kessner2 kessner3 novisit ///
  pretri2 pretri3, fe

*** 3.7.5 Allowing for different within- and between effects

egen mn_smok = mean(smoke), by(momid)
generate dev_smok = smoke - mn_smok

quietly xtset momid
xtreg birwt dev_smok mn_smok male mage hsgrad somecoll     ///
  collgrad married black kessner2 kessner3 novisit pretri2 pretri3, mle

lincom mn_smok - dev_smok

* Means for other time-varying covariates 
egen mn_male = mean(male), by(momid)
egen mn_mage = mean(mage), by(momid)
egen mn_kessner2 = mean(kessner2), by(momid)
egen mn_kessner3 = mean(kessner3), by(momid)
egen mn_novisit = mean(novisit), by(momid)
egen mn_pretri2 = mean(pretri2), by(momid)
egen mn_pretri3 = mean(pretri3), by(momid)

/*
foreach var of varlist male mage kessner* novisit pretri* {
   egen mn_`var' = mean(`var'), by(momid)
}
*/

quietly xtset momid
xtreg birwt smok mn_smok male mn_male mage mn_mage hsgrad somecoll   ///
  collgrad married black kessner2 mn_kessner2 kessner3 mn_kessner3   ///
  novisit mn_novisit pretri2 mn_pretri2 pretri3 mn_pretri3, mle

testparm mn_*


** 3.7.6 Hausman endogeneity test

quietly xtset momid
quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3, fe
estimates store fixed

quietly xtreg birwt smoke male mage hsgrad somecoll collgrad  ///
  married black kessner2 kessner3 novisit pretri2 pretri3, re
estimates store random

hausman fixed random


*** 3.9 Assigning values to random effects: Residual diagnostics

quietly xtmixed birwt smoke male mage hsgrad somecoll collgrad ///
   married black kessner2 kessner3 novisit pretri2 pretri3, ///
   || momid: , mle

predict lev2, reffects
predict comp_se, reses
generate diag_se = sqrt(exp(2*[lns1_1_1]_cons) - comp_se^2)
replace lev2 = lev2/diag_se
predict lev1, rstandard


histogram lev1, normal xtitle(Standardized level-1 residuals)

histogram lev2 if idx==1, normal xtitle(Standardized level-2 residuals)


quietly xtset momid
xtreg birwt smoke male mage hsgrad somecoll collgrad married black ///
   kessner2 kessner3 novisit pretri2 pretri3, vce(robust) re


