*****************************************************************
****Chapter 5: Subject-specific effects and dynamic models *****
*****************************************************************


*** 5.1 Introduction

use http://www.stata-press.com/data/mlmus3/wagepan, clear

generate educt = educ - 12
generate yeart = year - 1980


** 5.2 Conventional random-intercept model

xtmixed lwage black hisp union married exper yeart educt || nr:, mle
estimates store ri

** 5.3 Random-intercept models accommodating endogenous covariates

* 5.3.1 
egen mn_union = mean(union), by(nr)
egen mn_married = mean(married), by(nr)
egen mn_exper = mean(exper), by(nr)

generate dev_union = union - mn_union
generate dev_married = married - mn_married
generate dev_exper = exper - mn_exper

xtmixed lwage black hisp dev_union dev_married dev_exper educt ///
 mn_union mn_married mn_exper || nr:, mle vce(robust)
 
test (dev_union=mn_union) (dev_married=mn_married) (dev_exper=mn_exper)

lincom dev_union-mn_union
lincom dev_married-mn_married
lincom dev_exper-mn_exper

* 5.3.2
quietly xtset nr 
xthtaylor lwage black hisp union married exper educt, endog(union educt)

** 5.4 Fixed-intercept model

* 5.4.1 Using xtreg
quietly xtset nr 
xtreg lwage union married exper, fe
estimates store fi

*xtset nr year
*regress D.lwage D.union D.married 
*regress D.(lwage union married)

quietly xtset nr 
quietly xtreg lwage black hisp union married exper educt, re
estimates store ri2
hausman fi ri2

* 5.4.2 Using anova
generate ethnic = black*1 + hisp*2
set matsize 800
anova lwage ethnic / nr|ethnic union married c.exper, dropemptycells

display sqrt(18.63)
display sqrt(11.13)
display sqrt(536.97)

*anova lwage ethnic c.educt / nr|ethnic\#educt union married c.exper, dropemptycells


** 5.5 Random-coefficient model

xtmixed lwage black hisp union married exper yeart educt || nr: exper, ///
   covariance(unstructured) mle
estimates store rc

lrtest ri rc

** 5.6 Fixed-coefficient model

quietly xtset nr yeart
xtreg D.(lwage union married), fe
*regress D2.(lwage union married)

** 5.7 Lagged-response or dynamic models

* 5.7.2 Conventional lagged-response model
by nr (yeart), sort: generate lag1 = lwage[_n-1]

sort nr yeart
list nr yeart lwage lag1 in 1/9, clean noobs

regress lwage lag1 black hisp union married exper yeart educt

*xtset nr year
*regress lwage L.lwage black hisp union married exper yeart educt

* 5.7.2 Lagged-response models with subject-specific intercepts
quietly xtset nr yeart
ivregress 2sls D.lwage D.(union married) (LD.lwage = L2.lwage)

xtabond lwage union married exper, lags(1) twostep noconstant vce(robust)

** 5.8 Missing data and dropout

clear
set obs 100000
set seed 123123123
generate zeta = rnormal(0,1)
generate y1 = 2 + zeta + rnormal(0,1)
generate y2 = 2 + zeta + rnormal(0,1)

replace y2 = . if y1>2 & runiform()<0.9

tabstat y1 y2

generate id = _n
reshape long y, i(id) j(occasion)
quietly tabulate occasion, generate(occ)

quietly xtset id
xtreg y occ1 occ2, noconstant mle

regress y occ1 occ2, noconstant vce(cluster id)



